@@ -25,6 +25,8 @@
 #include "demos.h"
 #include "fontify.h"
 
+#include "demo_conf.h"
+
 static GtkWidget *info_view;
 static GtkWidget *source_view;
 
@@ -196,16 +198,20 @@ activate_about (GSimpleAction *action,
                           gtk_get_micro_version ());
   g_string_append_printf (s, "\nA link can appear here: <http://www.gtk.org>");
 
-  version = g_strdup_printf ("%s\nRunning against GTK %d.%d.%d",
+  version = g_strdup_printf ("%s%s%s\nRunning against GTK %d.%d.%d",
                              PACKAGE_VERSION,
+                             g_strcmp0 (PROFILE, "devel") == 0 ? "-" : "",
+                             g_strcmp0 (PROFILE, "devel") == 0 ? VCS_TAG : "",
                              gtk_get_major_version (),
                              gtk_get_minor_version (),
                              gtk_get_micro_version ());
 
   gtk_show_about_dialog (GTK_WINDOW (gtk_application_get_active_window (app)),
-                         "program-name", "GTK Demo",
+                         "program-name", g_strcmp0 (PROFILE, "devel") == 0
+                                         ? "GTK Demo (Development)"
+                                         : "GTK Demo",
                          "version", version,
-                         "copyright", "© 1997—2020 The GTK Team",
+                         "copyright", "© 1997—2021 The GTK Team",
                          "license-type", GTK_LICENSE_LGPL_2_1,
                          "website", "http://www.gtk.org",
                          "comments", "Program to demonstrate GTK widgets",
@@ -337,8 +343,8 @@ display_text (const char  *format,
 
   bytes = g_resources_lookup_data (resource, 0, NULL);
   g_assert (bytes);
-
-  g_assert (g_utf8_validate (g_bytes_get_data (bytes, NULL), g_bytes_get_size (bytes), NULL));
+  text = g_bytes_get_data (bytes, &len);
+  g_assert (g_utf8_validate (text, len, NULL));
 
   textview = gtk_text_view_new ();
   gtk_text_view_set_left_margin (GTK_TEXT_VIEW (textview), 20);
@@ -354,9 +360,8 @@ display_text (const char  *format,
   gtk_text_view_set_monospace (GTK_TEXT_VIEW (textview), TRUE);
 
   buffer = gtk_text_buffer_new (NULL);
-
-  text = g_bytes_unref_to_data (bytes, &len);
   gtk_text_buffer_set_text (buffer, text, len);
+  g_bytes_unref (bytes);
 
   if (format)
     fontify (format, buffer);
@@ -902,6 +907,9 @@ activate (GApplication *app)
   window = (GtkWidget *)gtk_builder_get_object (builder, "window");
   gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (window));
 
+  if (g_strcmp0 (PROFILE, "devel") == 0)
+    gtk_widget_add_css_class (window, "devel");
+
   action = g_simple_action_new ("run", NULL);
   g_signal_connect (action, "activate", G_CALLBACK (activate_run), window);
   g_action_map_add_action (G_ACTION_MAP (window), G_ACTION (action));
@@ -1044,10 +1052,10 @@ out:
 static void
 print_version (void)
 {
-  g_print ("gtk4-demo %d.%d.%d\n",
-           gtk_get_major_version (),
-           gtk_get_minor_version (),
-           gtk_get_micro_version ());
+  g_print ("gtk4-demo %s%s%s\n",
+           PACKAGE_VERSION,
+           g_strcmp0 (PROFILE, "devel") == 0 ? "-" : "",
+           g_strcmp0 (PROFILE, "devel") == 0 ? VCS_TAG : "");
 }
 
 static int
