@@ -21,6 +21,7 @@
 
 #include "gdkmacosglcontext-private.h"
 #include "gdkmacossurface-private.h"
+#include "gdkmacostoplevelsurface-private.h"
 
 #include "gdkinternals.h"
 #include "gdkintl.h"
@@ -100,6 +101,8 @@ create_pixel_format (int      major,
     NSOpenGLPFAAccelerated,
     NSOpenGLPFADoubleBuffer,
     NSOpenGLPFABackingStore,
+    NSOpenGLPFAColorSize, 24,
+    NSOpenGLPFAAlphaSize, 8,
 
     (NSOpenGLPixelFormatAttribute)nil
   };
@@ -133,7 +136,7 @@ ensure_gl_view (GdkMacosGLContext *self)
   nsview = _gdk_macos_surface_get_view (surface);
   nswindow = _gdk_macos_surface_get_native (surface);
 
-  if (!GDK_IS_MACOS_GL_VIEW (nsview))
+  if G_UNLIKELY (!GDK_IS_MACOS_GL_VIEW (nsview))
     {
       NSRect frame;
 
@@ -147,14 +150,14 @@ ensure_gl_view (GdkMacosGLContext *self)
 
       if (self->dummy_view != NULL)
         {
-          NSView *nsview = g_steal_pointer (&self->dummy_view);
-          [nsview release];
+          NSView *dummy_view = g_steal_pointer (&self->dummy_view);
+          [dummy_view release];
         }
 
       if (self->dummy_window != NULL)
         {
-          NSWindow *nswindow = g_steal_pointer (&self->dummy_window);
-          [nswindow release];
+          NSWindow *dummy_window = g_steal_pointer (&self->dummy_window);
+          [dummy_window release];
         }
     }
 
@@ -170,12 +173,13 @@ gdk_macos_gl_context_real_realize (GdkGLContext  *context,
   NSOpenGLContext *shared_gl_context = nil;
   NSOpenGLContext *gl_context;
   NSOpenGLPixelFormat *pixelFormat;
+  CGLContextObj cgl_context;
   GdkGLContext *shared;
   GdkGLContext *shared_data;
-  GdkGLContext *existing;
+  NSOpenGLContext *existing;
   GLint sync_to_framerate = 1;
-  GLint opaque = 0;
   GLint validate = 0;
+  GLint swapRect[4];
   int major, minor;
 
   g_assert (GDK_IS_MACOS_GL_CONTEXT (self));
@@ -183,7 +187,7 @@ gdk_macos_gl_context_real_realize (GdkGLContext  *context,
   if (self->gl_context != nil)
     return TRUE;
 
-  existing = gdk_gl_context_get_current ();
+  existing = [NSOpenGLContext currentContext];
 
   gdk_gl_context_get_required_version (context, &major, &minor);
 
@@ -224,9 +228,19 @@ gdk_macos_gl_context_real_realize (GdkGLContext  *context,
       return FALSE;
     }
 
-  [gl_context setValues:&sync_to_framerate forParameter:NSOpenGLCPSwapInterval];
-  [gl_context setValues:&opaque forParameter:NSOpenGLCPSurfaceOpacity];
-  [gl_context setValues:&validate forParameter:NSOpenGLContextParameterStateValidation];
+  cgl_context = [gl_context CGLContextObj];
+
+  swapRect[0] = 0;
+  swapRect[1] = 0;
+  swapRect[2] = surface->width;
+  swapRect[3] = surface->height;
+
+  CGLSetParameter (cgl_context, kCGLCPSwapRectangle, swapRect);
+  CGLSetParameter (cgl_context, kCGLCPSwapInterval, &sync_to_framerate);
+
+  CGLEnable (cgl_context, kCGLCESwapRectangle);
+  if (validate)
+    CGLEnable (cgl_context, kCGLCEStateValidation);
 
   self->dummy_window = [[NSWindow alloc] initWithContentRect:NSZeroRect
                                                    styleMask:0
@@ -248,19 +262,53 @@ gdk_macos_gl_context_real_realize (GdkGLContext  *context,
   self->gl_context = g_steal_pointer (&gl_context);
 
   if (existing != NULL)
-    [GDK_MACOS_GL_CONTEXT (existing)->gl_context makeCurrentContext];
+    [existing makeCurrentContext];
 
   return TRUE;
 }
 
+static gboolean
+opaque_region_covers_surface (GdkMacosGLContext *self)
+{
+  GdkSurface *surface;
+  cairo_region_t *region;
+
+  g_assert (GDK_IS_MACOS_GL_CONTEXT (self));
+
+  surface = gdk_draw_context_get_surface (GDK_DRAW_CONTEXT (self));
+  region = GDK_MACOS_SURFACE (surface)->opaque_region;
+
+  if (region != NULL &&
+      cairo_region_num_rectangles (region) == 1)
+    {
+      cairo_rectangle_int_t extents;
+
+      cairo_region_get_extents (region, &extents);
+
+      if (extents.x == 0 &&
+          extents.y == 0 &&
+          extents.width == surface->width &&
+          extents.height == surface->height)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 static void
 gdk_macos_gl_context_begin_frame (GdkDrawContext *context,
                                   cairo_region_t *painted)
 {
   GdkMacosGLContext *self = (GdkMacosGLContext *)context;
+  GdkSurface *surface;
 
   g_assert (GDK_IS_MACOS_GL_CONTEXT (self));
 
+  surface = gdk_draw_context_get_surface (context);
+
+  g_clear_pointer (&self->damage, cairo_region_destroy);
+  self->damage = cairo_region_copy (painted);
+
   /* If begin frame is called, that means we are trying to draw to
    * the NSWindow using our view. That might be a GdkMacosCairoView
    * but we need it to be a GL view. Also, only in this case do we
@@ -269,28 +317,38 @@ gdk_macos_gl_context_begin_frame (GdkDrawContext *context,
    */
   if (!self->is_attached &&
       gdk_gl_context_get_shared_context (GDK_GL_CONTEXT (context)))
-    {
-      ensure_gl_view (self);
-
-      g_clear_pointer (&self->damage, cairo_region_destroy);
-      self->damage = cairo_region_copy (painted);
-
-      cairo_region_get_extents (painted, &self->flush_rect);
-    }
+    ensure_gl_view (self);
 
   if (self->needs_resize)
     {
+      CGLContextObj cgl_context = [self->gl_context CGLContextObj];
+      GLint opaque;
+
       self->needs_resize = FALSE;
 
       if (self->dummy_view != NULL)
         {
-          GdkSurface *surface = gdk_draw_context_get_surface (context);
           NSRect frame = NSMakeRect (0, 0, surface->width, surface->height);
 
           [self->dummy_window setFrame:frame display:NO];
           [self->dummy_view setFrame:frame];
         }
 
+      /* Possibly update our opaque setting depending on a resize. We can
+       * rely on getting a resize if decoarated is changed, so this reduces
+       * how much we adjust the parameter.
+       */
+      if (GDK_IS_MACOS_TOPLEVEL_SURFACE (surface))
+        opaque = GDK_MACOS_TOPLEVEL_SURFACE (surface)->decorated;
+      else
+        opaque = FALSE;
+
+      /* If we are maximized, we might be able to make it opaque */
+      if (opaque == FALSE)
+        opaque = opaque_region_covers_surface (self);
+
+      CGLSetParameter (cgl_context, kCGLCPSurfaceOpacity, &opaque);
+
       [self->gl_context update];
     }
 
@@ -298,8 +356,7 @@ gdk_macos_gl_context_begin_frame (GdkDrawContext *context,
 
   if (!self->is_attached)
     {
-      GdkMacosSurface *surface = GDK_MACOS_SURFACE (gdk_draw_context_get_surface (context));
-      NSView *nsview = _gdk_macos_surface_get_view (surface);
+      NSView *nsview = _gdk_macos_surface_get_view (GDK_MACOS_SURFACE (surface));
 
       g_assert (self->gl_context != NULL);
       g_assert (GDK_IS_MACOS_GL_VIEW (nsview));
@@ -313,29 +370,32 @@ gdk_macos_gl_context_end_frame (GdkDrawContext *context,
                                 cairo_region_t *painted)
 {
   GdkMacosGLContext *self = GDK_MACOS_GL_CONTEXT (context);
-  GdkMacosSurface *surface;
-  NSView *nsview;
-  cairo_rectangle_int_t extents;
 
   g_assert (GDK_IS_MACOS_GL_CONTEXT (self));
   g_assert (self->gl_context != nil);
 
-  surface = GDK_MACOS_SURFACE (gdk_draw_context_get_surface (context));
-  nsview = self->dummy_view ?
-           self->dummy_view :
-           _gdk_macos_surface_get_view (surface);
-
   GDK_DRAW_CONTEXT_CLASS (gdk_macos_gl_context_parent_class)->end_frame (context, painted);
 
-  /* We want to limit how much gets moved to the front buffer so here
-   * we adjust the clip rectangle before flushBuffer is called.
-   */
-  G_STATIC_ASSERT (sizeof (GLint) == sizeof (int));
-  [self->gl_context
-         setValues:(GLint *)&self->flush_rect
-      forParameter:NSOpenGLCPSwapRectangle];
+  if (!self->is_attached)
+    {
+      GdkSurface *surface = gdk_draw_context_get_surface (context);
+      CGLContextObj glctx = [self->gl_context CGLContextObj];
+      cairo_rectangle_int_t flush_rect;
+      GLint swapRect[4];
 
-  [self->gl_context flushBuffer];
+      /* Coordinates are in display coordinates, where as flush_rect is
+       * in GDK coordinates. Must flip Y to match display coordinates where
+       * 0,0 is the bottom-left corner.
+       */
+      cairo_region_get_extents (painted, &flush_rect);
+      swapRect[0] = flush_rect.x;                   /* left */
+      swapRect[1] = surface->height - flush_rect.y; /* bottom */
+      swapRect[2] = flush_rect.width;               /* width */
+      swapRect[3] = flush_rect.height;              /* height */
+      CGLSetParameter (glctx, kCGLCPSwapRectangle, swapRect);
+
+      [self->gl_context flushBuffer];
+    }
 }
 
 static void
@@ -442,15 +502,28 @@ _gdk_macos_gl_context_new (GdkMacosSurface  *surface,
 gboolean
 _gdk_macos_gl_context_make_current (GdkMacosGLContext *self)
 {
+  NSOpenGLContext *current;
+
   g_return_val_if_fail (GDK_IS_MACOS_GL_CONTEXT (self), FALSE);
 
-  if (self->gl_context != nil)
+  if (self->gl_context == NULL)
+    return FALSE;
+
+  current = [NSOpenGLContext currentContext];
+
+  if (self->gl_context != current)
     {
+      /* The OpenGL mac programming guide suggests that glFlush() is called
+       * before switching current contexts to ensure that the drawing commands
+       * are submitted.
+       */
+      if (current != NULL)
+        glFlush ();
+
       [self->gl_context makeCurrentContext];
-      return TRUE;
     }
 
-  return FALSE;
+  return TRUE;
 }
 
 G_GNUC_END_IGNORE_DEPRECATIONS
