@@ -94,7 +94,7 @@
 
 #define MIN_SYSTEM_BELL_DELAY_MS 20
 
-#define GTK_SHELL1_VERSION       3
+#define GTK_SHELL1_VERSION       4
 #define OUTPUT_VERSION_WITH_DONE 2
 #define NO_XDG_OUTPUT_DONE_SINCE_VERSION 3
 
@@ -339,6 +339,48 @@ get_format_name (enum wl_shm_format format)
     FORMAT(YVU422),
     FORMAT(YUV444),
     FORMAT(YVU444),
+    FORMAT(R8),
+    FORMAT(R16),
+    FORMAT(RG88),
+    FORMAT(GR88),
+    FORMAT(RG1616),
+    FORMAT(GR1616),
+    FORMAT(XRGB16161616F),
+    FORMAT(XBGR16161616F),
+    FORMAT(ARGB16161616F),
+    FORMAT(ABGR16161616F),
+    FORMAT(XYUV8888),
+    FORMAT(VUY888),
+    FORMAT(VUY101010),
+    FORMAT(Y210),
+    FORMAT(Y212),
+    FORMAT(Y216),
+    FORMAT(Y410),
+    FORMAT(Y412),
+    FORMAT(Y416),
+    FORMAT(XVYU12_16161616),
+    FORMAT(XVYU16161616),
+    FORMAT(Y0L0),
+    FORMAT(X0L0),
+    FORMAT(Y0L2),
+    FORMAT(X0L2),
+    FORMAT(YUV420_8BIT),
+    FORMAT(YUV420_10BIT),
+    FORMAT(XRGB8888_A8),
+    FORMAT(XBGR8888_A8),
+    FORMAT(RGBX8888_A8),
+    FORMAT(BGRX8888_A8),
+    FORMAT(RGB888_A8),
+    FORMAT(BGR888_A8),
+    FORMAT(RGB565_A8),
+    FORMAT(BGR565_A8),
+    FORMAT(NV24),
+    FORMAT(NV42),
+    FORMAT(P210),
+    FORMAT(P010),
+    FORMAT(P012),
+    FORMAT(P016),
+
     { 0xffffffff, NULL }
   };
 #undef FORMAT
@@ -1350,21 +1392,23 @@ _gdk_wayland_display_create_shm_surface (GdkWaylandDisplay *display,
   data->buffer = NULL;
   data->scale = scale;
 
-  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, width*scale);
+  stride = cairo_format_stride_for_width (CAIRO_FORMAT_ARGB32, width * scale);
 
   data->pool = create_shm_pool (display->shm,
-                                height*scale*stride,
+                                height * scale * stride,
                                 &data->buf_length,
                                 &data->buf);
+  if (G_UNLIKELY (data->pool == NULL))
+    g_error ("Unable to create shared memory pool");
 
   surface = cairo_image_surface_create_for_data (data->buf,
                                                  CAIRO_FORMAT_ARGB32,
-                                                 width*scale,
-                                                 height*scale,
+                                                 width * scale,
+                                                 height * scale,
                                                  stride);
 
   data->buffer = wl_shm_pool_create_buffer (data->pool, 0,
-                                            width*scale, height*scale,
+                                            width * scale, height * scale,
                                             stride, WL_SHM_FORMAT_ARGB8888);
 
   cairo_surface_set_user_data (surface, &gdk_wayland_shm_surface_cairo_key,
@@ -1546,15 +1590,11 @@ update_xft_settings (GdkDisplay *display)
     }
   else
     {
-      GSettingsSchemaSource *source;
-      GSettingsSchema *schema;
+      TranslationEntry *entry;
 
-      source = g_settings_schema_source_get_default ();
-      schema = g_settings_schema_source_lookup (source,
-                                                "org.gnome.desktop.interface",
-                                                FALSE);
+      entry = find_translation_entry_by_schema ("org.gnome.desktop.interface", "font-antialiasing");
 
-      if (schema && g_settings_schema_has_key (schema, "font-antialiasing"))
+      if (entry && entry->valid)
         {
           settings = g_hash_table_lookup (display_wayland->settings,
                                           "org.gnome.desktop.interface");
@@ -1578,9 +1618,6 @@ update_xft_settings (GdkDisplay *display)
           order = GSD_FONT_RGBA_ORDER_RGB;
         }
 
-      if (schema)
-        g_settings_schema_unref (schema);
-
       dpi = get_dpi_from_gsettings (display_wayland) * 1024;
     }
 
