@@ -39,7 +39,7 @@
 #include "gtkadjustment.h"
 #include "gtkgesturedrag.h"
 #include "gtkeventcontrollermotion.h"
-#include "gtkdragsource.h"
+#include "gtkdragsourceprivate.h"
 #include "gtkeventcontrollerkey.h"
 #include "gtkgestureclick.h"
 
@@ -983,6 +983,8 @@ header_drag_begin (GtkGestureDrag *gesture,
 
           break;
         }
+
+      g_object_unref (column);
     }
 
   for (i = 0; !self->in_column_resize && i < n; i++)
@@ -1049,7 +1051,10 @@ header_drag_end (GtkGestureDrag *gesture,
 
       sequence = gtk_gesture_single_get_current_sequence (GTK_GESTURE_SINGLE (gesture));
       if (!gtk_gesture_handles_sequence (GTK_GESTURE (gesture), sequence))
-        return;
+        {
+          g_object_unref (column);
+          return;
+        }
 
       for (i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (self->columns)); i++)
         {
@@ -1123,7 +1128,7 @@ header_drag_update (GtkGestureDrag *gesture,
 
   if (!self->in_column_resize && !self->in_column_reorder)
     {
-      if (gtk_drag_check_threshold (GTK_WIDGET (self), 0, 0, offset_x, 0))
+      if (gtk_drag_check_threshold_double (GTK_WIDGET (self), 0, 0, offset_x, 0))
         {
           GtkColumnViewColumn *column;
           GtkWidget *header;
@@ -1531,7 +1536,6 @@ gtk_column_view_remove_column (GtkColumnView       *self,
       g_object_unref (item);
       if (item == column)
         break;
-
     }
 
   gtk_column_view_sorter_remove_column (GTK_COLUMN_VIEW_SORTER (self->sorter), column);
@@ -1657,7 +1661,7 @@ gtk_column_view_get_list_view (GtkColumnView *self)
  *   gtk_column_view_set_model (view, selection);
  * ]|
  *
- * Returns: (transfer none): the #GtkSorter of @self
+ * Returns: (nullable) (transfer none): the #GtkSorter of @self
  */
 GtkSorter *
 gtk_column_view_get_sorter (GtkColumnView *self)
