@@ -22,6 +22,9 @@
 
 #include "gtkexpression.h"
 
+/* XXX: For g_memdup2() */
+#include "gtkprivate.h"
+
 #include <gobject/gvaluecollector.h>
 
 /**
@@ -249,7 +252,7 @@ struct _GtkExpressionWatch
 GType \
 type_name ## _get_type (void) \
 { \
-  static volatile gsize gtk_define_expression_type_id__volatile; \
+  static gsize gtk_define_expression_type_id__volatile; \
   if (g_once_init_enter (&gtk_define_expression_type_id__volatile)) \
     { \
       GType gtk_define_expression_type_id = \
@@ -490,7 +493,7 @@ param_expression_values_cmp (GParamSpec   *pspec,
 GType
 gtk_param_expression_get_type (void)
 {
-  static volatile gsize param_expression_type__volatile;
+  static gsize param_expression_type__volatile;
 
   if (g_once_init_enter (&param_expression_type__volatile))
     {
@@ -598,7 +601,7 @@ gtk_expression_init (GtkExpression *self)
 GType
 gtk_expression_get_type (void)
 {
-  static volatile gsize expression_type__volatile;
+  static gsize expression_type__volatile;
 
   if (g_once_init_enter (&expression_type__volatile))
     {
@@ -685,7 +688,7 @@ gtk_expression_type_register_static (const char                  *type_name,
   info.base_finalize = NULL;
   info.class_init = gtk_expression_generic_class_init;
   info.class_finalize = NULL;
-  info.class_data = g_memdup (type_info, sizeof (GtkExpressionTypeInfo));
+  info.class_data = g_memdup2 (type_info, sizeof (GtkExpressionTypeInfo));
 
   info.instance_size = type_info->instance_size;
   info.n_preallocs = 0;
