@@ -37,7 +37,7 @@
 #include "gtkgesturelongpressprivate.h"
 #include "gtkgestureprivate.h"
 #include "gtkmarshalers.h"
-#include "gtkdragsource.h"
+#include "gtkdragsourceprivate.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
@@ -51,7 +51,8 @@ enum {
 };
 
 enum {
-  PROP_DELAY_FACTOR = 1
+  PROP_DELAY_FACTOR = 1,
+  LAST_PROP
 };
 
 struct _GtkGestureLongPressPrivate
@@ -67,15 +68,14 @@ struct _GtkGestureLongPressPrivate
 };
 
 static guint signals[N_SIGNALS] = { 0 };
+static GParamSpec *props[LAST_PROP] = { NULL, };
 
 G_DEFINE_TYPE_WITH_PRIVATE (GtkGestureLongPress, gtk_gesture_long_press, GTK_TYPE_GESTURE_SINGLE)
 
 static void
 gtk_gesture_long_press_init (GtkGestureLongPress *gesture)
 {
-  GtkGestureLongPressPrivate *priv;
-
-  priv = gtk_gesture_long_press_get_instance_private (GTK_GESTURE_LONG_PRESS (gesture));
+  GtkGestureLongPressPrivate *priv = gtk_gesture_long_press_get_instance_private (gesture);
   priv->delay_factor = 1.0;
 }
 
@@ -159,7 +159,7 @@ gtk_gesture_long_press_update (GtkGesture       *gesture,
   priv = gtk_gesture_long_press_get_instance_private (GTK_GESTURE_LONG_PRESS (gesture));
   gtk_gesture_get_point (gesture, sequence, &x, &y);
 
-  if (gtk_drag_check_threshold (widget, priv->initial_x, priv->initial_y, x, y))
+  if (gtk_drag_check_threshold_double (widget, priv->initial_x, priv->initial_y, x, y))
     {
       if (priv->timeout_id)
         {
@@ -275,13 +275,14 @@ gtk_gesture_long_press_class_init (GtkGestureLongPressClass *klass)
   gesture_class->cancel = gtk_gesture_long_press_cancel;
   gesture_class->sequence_state_changed = gtk_gesture_long_press_sequence_state_changed;
 
-  g_object_class_install_property (object_class,
-                                   PROP_DELAY_FACTOR,
-                                   g_param_spec_double ("delay-factor",
-                                                        P_("Delay factor"),
-                                                        P_("Factor by which to modify the default timeout"),
-                                                        0.5, 2.0, 1.0,
-                                                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY));
+  props[PROP_DELAY_FACTOR] =
+    g_param_spec_double ("delay-factor",
+                         P_("Delay factor"),
+                         P_("Factor by which to modify the default timeout"),
+                         0.5, 2.0, 1.0,
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  g_object_class_install_properties (object_class, LAST_PROP, props);
 
   /**
    * GtkGestureLongPress::pressed:
@@ -351,9 +352,12 @@ gtk_gesture_long_press_set_delay_factor (GtkGestureLongPress *gesture,
   g_return_if_fail (delay_factor >= 0.5);
   g_return_if_fail (delay_factor <= 2.0);
 
+  if (delay_factor == priv->delay_factor)
+    return;
+
   priv->delay_factor = delay_factor;
 
-  g_object_notify (G_OBJECT (gesture), "delay-factor");
+  g_object_notify_by_pspec (G_OBJECT (gesture), props[PROP_DELAY_FACTOR]);
 }
 
 /**
