@@ -21,6 +21,7 @@
 #include <string.h>
 #include <wayland-client-protocol.h>
 
+#include "gtk/gtkdragsourceprivate.h"
 #include "gtk/gtkimcontextwayland.h"
 #include "gtk/gtkintl.h"
 #include "gtk/gtkimmoduleprivate.h"
@@ -524,10 +525,10 @@ released_cb (GtkGestureClick     *gesture,
   if (global->focused &&
       n_press == 1 &&
       (hints & GTK_INPUT_HINT_INHIBIT_OSK) == 0 &&
-      !gtk_drag_check_threshold (context->widget,
-                                 context->press_x,
-                                 context->press_y,
-                                 x, y))
+      !gtk_drag_check_threshold_double (context->widget,
+                                        context->press_x,
+                                        context->press_y,
+                                        x, y))
     {
       zwp_text_input_v3_enable (global->text_input);
       g_signal_emit_by_name (global->current, "retrieve-surrounding", &result);
@@ -546,6 +547,7 @@ gtk_im_context_wayland_set_client_widget (GtkIMContext *context,
 
   if (context_wayland->widget)
     {
+      gtk_im_context_wayland_focus_out (context);
       gtk_widget_remove_controller (context_wayland->widget, GTK_EVENT_CONTROLLER (context_wayland->gesture));
       context_wayland->gesture = NULL;
     }
@@ -568,6 +570,38 @@ gtk_im_context_wayland_set_client_widget (GtkIMContext *context,
     }
 }
 
+/* We want a unified experience between GtkIMContextSimple and IBus / Wayland
+ * when it comes to Compose sequences. IBus initial implementation of preedit
+ * for Compose sequences shows U+2384, which has been described as 'distracting'.
+ * This function tries to detect this case, and tweaks the text to match what
+ * GtkIMContextSimple produces.
+ */
+static char *
+tweak_preedit (const char *text)
+{
+  GString *s;
+  guint len;
+
+  s = g_string_new ("");
+
+  len = g_utf8_strlen (text, -1);
+
+  for (const char *p = text; *p; p = g_utf8_next_char (p))
+    {
+      gunichar ch = g_utf8_get_char (p);
+
+      if (ch == 0x2384)
+        {
+          if (len == 1 || p > text)
+            g_string_append (s, "·");
+        }
+      else
+        g_string_append_unichar (s, ch);
+    }
+
+  return g_string_free (s, FALSE);
+}
+
 static void
 gtk_im_context_wayland_get_preedit_string (GtkIMContext   *context,
                                            char          **str,
@@ -575,7 +609,7 @@ gtk_im_context_wayland_get_preedit_string (GtkIMContext   *context,
                                            int            *cursor_pos)
 {
   GtkIMContextWayland *context_wayland = GTK_IM_CONTEXT_WAYLAND (context);
-  const char *preedit_str;
+  char *preedit_str;
 
   if (attrs)
     *attrs = NULL;
@@ -594,20 +628,31 @@ gtk_im_context_wayland_get_preedit_string (GtkIMContext   *context,
     }
 
   preedit_str =
-    context_wayland->current_preedit.text ? context_wayland->current_preedit.text : "";
+    tweak_preedit (context_wayland->current_preedit.text ? context_wayland->current_preedit.text : "");
 
-  if (str)
-    *str = g_strdup (preedit_str);
   if (cursor_pos)
     *cursor_pos = g_utf8_strlen (preedit_str,
                                  context_wayland->current_preedit.cursor_begin);
 
   if (attrs)
     {
+      PangoAttribute *attr;
+      guint len = strlen (preedit_str);
+
       if (!*attrs)
         *attrs = pango_attr_list_new ();
-      pango_attr_list_insert (*attrs,
-                              pango_attr_underline_new (PANGO_UNDERLINE_SINGLE));
+
+      attr = pango_attr_underline_new (PANGO_UNDERLINE_SINGLE);
+      attr->start_index = 0;
+      attr->end_index = len;
+      pango_attr_list_insert (*attrs, attr);
+
+      /* enable fallback, since IBus will send us things like ⎄ */
+      attr = pango_attr_fallback_new (TRUE);
+      attr->start_index = 0;
+      attr->end_index = len;
+      pango_attr_list_insert (*attrs, attr);
+
       if (context_wayland->current_preedit.cursor_begin
           != context_wayland->current_preedit.cursor_end)
         {
@@ -618,6 +663,10 @@ gtk_im_context_wayland_get_preedit_string (GtkIMContext   *context,
           pango_attr_list_insert (*attrs, cursor);
         }
     }
+  if (str)
+    *str = preedit_str;
+  else
+    g_free (preedit_str);
 }
 
 static gboolean
