@@ -52,6 +52,11 @@
  * various common properties on the dialog, as well as show and hide
  * it and get a #GtkNativeDialog::response signal when the user finished
  * with the dialog.
+ *
+ * Note that unlike #GtkDialog, #GtkNativeDialog objects are not
+ * toplevel widgets, and GTK does not keep them alive. It is your
+ * responsibility to keep a reference until you are done with the
+ * object.
  */
 
 typedef struct _GtkNativeDialogPrivate GtkNativeDialogPrivate;
@@ -63,10 +68,6 @@ struct _GtkNativeDialogPrivate
 
   guint visible : 1;
   guint modal : 1;
-
-  /* Run state */
-  int run_response_id;
-  GMainLoop *run_loop; /* Non-NULL when in run */
 };
 
 enum {
@@ -342,9 +343,6 @@ gtk_native_dialog_hide (GtkNativeDialog *self)
 
   klass->hide (self);
 
-  if (priv->run_loop && g_main_loop_is_running (priv->run_loop))
-    g_main_loop_quit (priv->run_loop);
-
   g_object_notify_by_pspec (G_OBJECT (self), native_props[PROP_VISIBLE]);
 }
 
