@@ -93,7 +93,7 @@ static void
 do_delete (gpointer     funcs_data,
            guint        begin,
            guint        end,
-           const gchar *expected_text,
+           const char *expected_text,
            guint        len)
 {
   Text *t = funcs_data;
@@ -126,8 +126,8 @@ do_delete (gpointer     funcs_data,
 
 static void
 do_select (gpointer funcs_data,
-           gint     selection_insert,
-           gint     selection_bound)
+           int      selection_insert,
+           int      selection_bound)
 {
   Text *text = funcs_data;
 
