@@ -2,7 +2,7 @@
 #include <glib/gstdio.h>
 #include <gtk/gtk.h>
 #include <stdlib.h>
-#include "reftest-compare.h"
+#include "../reftests/reftest-compare.h"
 
 static char *arg_output_dir = NULL;
 
